require 'nokogiri'
require 'classprop'
require 'standarderror/plus'
require 'uri'


#===============================================================================
# Motley
#
module Motley
	# version
	VERSION = '0.1'
end
#
# Motley
#===============================================================================


#===============================================================================
# Motley::Site
#
class Motley::Site
	attr_accessor :root
	attr_accessor :resources
	
	#---------------------------------------------------------------------------
	# initialize
	#
	def initialize(root, resources, opts={})
		@root = root
		@resources = resources
	end
	#
	# initialize
	#---------------------------------------------------------------------------
end
#
# Motley::Site
#===============================================================================


#===============================================================================
# Motley::AttSet
#
class Motley::AttSet
	extend Forwardable
	attr_reader :dfns
	attr_reader :vals
	
	# delegate
	delegate %w([] each length clear delete keys values) => :vals
	
	
	#---------------------------------------------------------------------------
	# initialize
	#
	def initialize(dfns)
		@dfns = dfns
		@vals = {}
	end
	#
	# initialize
	#---------------------------------------------------------------------------
	
	
	#---------------------------------------------------------------------------
	# []=
	#
	def []=(key, val)
		# $tm.hrm
		
		# normalize key
		key = key.downcase
		
		# get att definition
		dfn = @dfns[key]
		
		# if no definition
		if not dfn
			exc = StandardError::Plus.new('/zax/attset/no-att-definition')
			exc['att'] = key
			raise exc
		end
		
		# XML and HTML do not have the concept of null, so if nil is assigned,
		# we delete the hash element. Else we set the value
		if val.nil?
			@vals.delete(key)
		else
			@vals[key] = dfn.set(val)
		end
	end
	#
	# []=
	#---------------------------------------------------------------------------
end
#
# Motley::AttSet
#===============================================================================


#===============================================================================
# Motley::AttDef
#
class Motley::AttDef
	attr_accessor :set_once
	
	#---------------------------------------------------------------------------
	# initialize
	#
	def initialize
		@set_once = false
	end
	#
	# initialize
	#---------------------------------------------------------------------------
	
	
	#---------------------------------------------------------------------------
	# to_str
	#
	def to_str
		raise 'zax/attdef/default/override-to-s'
	end
	#
	# to_str
	#---------------------------------------------------------------------------
end
#
# Motley::AttDef
#===============================================================================


#===============================================================================
# Motley::AttDef::Scalar
#
class Motley::AttDef::Scalar < Motley::AttDef
	ALLOWED = [String, Integer, Float, TrueClass, FalseClass]
	
	
	#---------------------------------------------------------------------------
	# set
	#
	def set(val)
		allowed_class(val)
		return val
	end
	#
	# set
	#---------------------------------------------------------------------------
	
	
	#---------------------------------------------------------------------------
	# allowed_class
	#
	def allowed_class(val)
		# loop through allowed classes
		ALLOWED.each do |allowed|
			if val.is_a?(allowed)
				return true
			end
		end
		
		# raise exception
		raise 'zax/attdef/scalar/allowed-class: ' + ALLOWED.to_s
	end
	#
	# allowed_class
	#---------------------------------------------------------------------------
	
	
	#---------------------------------------------------------------------------
	# to_str
	#
	def to_str(val)
		return val.to_s
	end
	#
	# to_str
	#---------------------------------------------------------------------------
end
#
# Motley::AttDef::Scalar
#===============================================================================


#===============================================================================
# Motley::AttDef::URI
#
class Motley::AttDef::URI < Motley::AttDef
	#---------------------------------------------------------------------------
	# set
	#
	def set(val)
		return URI(val)
	end
	#
	# set
	#---------------------------------------------------------------------------
	
	
	#---------------------------------------------------------------------------
	# to_str
	#
	def to_str(val)
		return val.to_s
	end
	#
	# to_str
	#---------------------------------------------------------------------------
end
#
# Motley::AttDef::URI
#===============================================================================


#===============================================================================
# Motley::TagBase
#
class Motley::TagBase
	attr_reader :atts
	include ClassProp
	define_class_prop 'att_defs', 'inherit'=>false
	
	#---------------------------------------------------------------------------
	# initialize
	#
	def initialize
		@atts = Motley::AttSet.new(self.class.att_defs)
	end
	#
	# initialize
	#---------------------------------------------------------------------------
end
#
# Motley::TagBase
#===============================================================================


#===============================================================================
# Motley::Tag
#
class Motley::Tag < Motley::TagBase
	attr_reader :children
	
	# delegate
	extend Forwardable
	delegate %w([] []= each length keys values) => :atts
	
	#---------------------------------------------------------------------------
	# initialize
	#
	def initialize
		# $tm.hrm
		super()
		@atts = Motley::AttSet.new(self.class.att_defs)
		@children = []
	end
	#
	# initialize
	#---------------------------------------------------------------------------
	
	
	#---------------------------------------------------------------------------
	# set_atts
	#
	def self.set_atts
		# initialize hash
		if not self.att_defs
			ads = self.att_defs = {}
			ads['id'] = Motley::AttDef::Scalar.new
		end
		
		# yield
		if block_given?
			yield self.att_defs
		end
	end
	#
	# set_atts
	#---------------------------------------------------------------------------
end
#
# Motley::Tag
#===============================================================================



#===============================================================================
# Motley::Tag::Nested
#
class Motley::Tag::Nested < Motley::Tag
	attr_accessor :parent
	
	#---------------------------------------------------------------------------
	# initialize
	#
	def initialize(parent=nil)
		super()
		@parent = parent
	end
	#
	# initialize
	#---------------------------------------------------------------------------
end
#
# Motley::Tag::Nested
#===============================================================================


#===============================================================================
# Motley::Document
#
class Motley::Document
	attr_reader :root
	
	#---------------------------------------------------------------------------
	# initialize
	#
	def initialize(opts={})
		# root
		@root = Motley::Tag::HTML.new(self)
	end
	#
	# initialize
	#---------------------------------------------------------------------------
end
#
# Motley::Document
#===============================================================================


#===============================================================================
# Motley::Schema
#
class Motley::Schema
	attr_reader :tags
	
	#---------------------------------------------------------------------------
	# initialize
	#
	def initialize()
		@tags = {}
	end
	#
	# initialize
	#---------------------------------------------------------------------------
end
#
# Motley::Schema
#===============================================================================


#===============================================================================
# Motley::Schema
#===============================================================================


#===============================================================================
# Motley::SchemaSet
#
class Motley::SchemaSet
	attr_reader :schemas
	attr_reader :tags
	
	# delegate
	extend Forwardable
	delegate %w([] []= each push pop shift unshift length) => :schemas

	
	#---------------------------------------------------------------------------
	# initialize
	#
	def initialize
		@schemas = []
		@tags = Motley::SchemaSet::TagFinder.new(self)
	end
	#
	# initialize
	#---------------------------------------------------------------------------
end
#
# Motley::SchemaSet
#===============================================================================


#===============================================================================
# Motley::SchemaSet::TagFinder
#
class Motley::SchemaSet::TagFinder
	attr_reader :schemaset
	
	
	#---------------------------------------------------------------------------
	# initialize
	#
	def initialize(schemaset)
		@schemaset = schemaset
	end
	#
	# initialize
	#---------------------------------------------------------------------------
	
	
	#---------------------------------------------------------------------------
	# []
	#
	def [](key)
		# $tm.hrm
		
		# normalize key
		key = key.downcase
		
		# loop through schemas looking for tag
		@schemaset.each do |schema|
			if tag = schema.tags[key]
				return tag
			end
		end
		
		# didn't find the tag
		return nil
	end
	#
	# []
	#---------------------------------------------------------------------------
end
#
# Motley::SchemaSet::TagFinder
#===============================================================================


# html
require 'motley/schemas/html'
require 'motley/schemas/mml'